/* Copyright 2019 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================*/

package quake.quake_tensorflow_face_rec.detection.tflite;

import android.graphics.Bitmap;
import android.graphics.RectF;

import java.util.List;

/**
 * Generic interface for interacting with different recognition engines.
 */
//与不同识别引擎交互的通用接口
//此接口从Classifier重命名为SimilarityClassifier
public interface SimilarityClassifier {
    //添加了将样本注册到数据集中的方法
    void register(String name, Recognition recognition);

    List<Recognition> recognizeImage(Bitmap bitmap, String registerOrRecognize,int frameNum);

    void enableStatLogging(final boolean debug);

    String getStatString();

    void close();

    void setNumThreads(int num_threads);

    void setUseNNAPI(boolean isChecked);

    /**
     * An immutable result returned by a Classifier describing what was recognized.
     */
    //分类器返回一个不可变的结果，用来描述所识别的内容
    public class Recognition {
        /**
         * A unique identifier for what has been recognized. Specific to the class, not the instance of
         * the object.
         */
        //已识别内容的唯一标识符。特定于类，而不是对象的实例
        private final String id;

        /**
         * Display name for the recognition.
         */
        //显示识别的名字
        private final String title;

        /**
         * A sortable score for how good the recognition is relative to others. Lower should be better.
         */
        //一个可排序的分数，用来衡量识别能力相对于其他人的程度，越低越好
        //我们使用距离而不是置信度
        private final Float distance;
        //extra字段用来存储任何数据（我们将使用它来存储embeddings识别（嵌入量））
        private Object extra;
        private float[][] embeddings;
        private int count;

        public String getLOG() {
            return LOG;
        }

        public void setLOG(String LOG) {
            this.LOG = LOG;
        }

        private String LOG;
        /**
         * Optional location within the source image for the location of the recognized object.
         */
        //源图像中用于识别对象位置的可选位置
        private RectF location;
        private Integer color;
        private Bitmap crop;

        public Recognition(
                final String id, final String title, final Float distance, final RectF location) {
            this.id = id;
            this.title = title;
            this.distance = distance;
            this.location = location;
            this.color = null;
            this.extra = null;
            this.embeddings = null;
            this.crop = null;
            this.LOG = null;
        }

        public void setExtra(Object extra) {
            this.extra = extra;
        }

        public Object getExtra() {
            return this.extra;
        }

        public void setCount(int count) {
            this.count = count;
        }

        public int getCount() {
            return this.count;
        }

        public void setEmbeddings(float[][] embeddings) {
            this.embeddings = embeddings;
        }

        public float[][] getEmbeddings() {
            return this.embeddings;
        }

        public void setColor(Integer color) {
            this.color = color;
        }

        public String getId() {
            return id;
        }

        public String getTitle() {
            return title;
        }

        public Float getDistance() {
            return distance;
        }

        public RectF getLocation() {
            return new RectF(location);
        }

        public void setLocation(RectF location) {
            this.location = location;
        }

        @Override
        public String toString() {
            String resultString = "";
            if (id != null) {
                resultString += "[" + id + "] ";
            }

            if (title != null) {
                resultString += title + " ";
            }

            if (distance != null) {
//                resultString += String.format("(%.1f%%) ", distance * 100.0f);
                resultString +=  distance +" ";
            }

            if (location != null) {
                resultString += location + " ";
            }

            return resultString.trim();
        }

        public Integer getColor() {
            return this.color;
        }

        public void setCrop(Bitmap crop) {
            this.crop = crop;
        }

        public Bitmap getCrop() {
            return this.crop;
        }
    }
}
